'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _map = require('babel-runtime/core-js/map');

var _map2 = _interopRequireDefault(_map);

exports.default = function (array) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};


    if (!Array.isArray(array)) {
        return [];
    }

    var _getUniques = getUniques(array, options),
        uniques = _getUniques.uniques,
        duplicates = _getUniques.duplicates;

    if (options.inplace) {
        array.length = 0;
        array.push.apply(array, uniques);
        uniques = array;
    }

    if (typeof options.onUnique === 'function') {
        for (var i = 0; i < uniques.length; i++) {
            var el = uniques[i];
            options.onUnique(el, duplicates.get(el), i, uniques);
        }
    }

    return uniques;
};

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isEqualWith = require('lodash.isequalwith');

/**
 * @param {Object} [options]
 * @param {Boolean} [options.sorted = false]
 * @param {Boolean} [options.inplace = false]
 * @param {Boolean} [options.strict = true]
 * @param {String|Function} [options.compare]
 * @param {String|Array(String)|Array(Array(String))} [options.by]
 * @param {String|Array(String)|Array(Array(String))} [options.skip]
 * @param {Function} [options.onUnique]
 */

function getUniques(array, options) {

    var uniques = [];

    if (!array.length) {
        return uniques;
    }

    var contains = options.sorted ? containsSorted : containsRandom;
    var equals = getEquals(options);

    var trackDuplicates = typeof options.onUnique === 'function';

    var duplicates = void 0;

    if (trackDuplicates) {
        duplicates = new _map2.default();
    }

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = (0, _getIterator3.default)(array), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var el = _step.value;


            var index = contains(uniques, el, equals, options);

            if (index === -1) {

                if (trackDuplicates) {
                    duplicates.set(el, []);
                }

                uniques.push(el);
            } else if (trackDuplicates) {
                duplicates.get(uniques[index]).push(el);
            }
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return { uniques: uniques, duplicates: duplicates };
}

function containsSorted(array, el, equals, options) {

    var lastIndex = array.length - 1;

    if (lastIndex !== -1) {

        var lastEl = array[lastIndex];

        if (equals(lastEl, el, options)) {
            return lastIndex;
        }
    }

    return -1;
}

function containsRandom(array, el, equals, options) {

    for (var i = 0; i < array.length; i++) {

        var unique = array[i];

        if (equals(unique, el, options)) {
            return i;
        }
    }

    return -1;
}

function getEquals(options) {

    if (typeof options.compare === 'function') {
        return options.compare;
    }

    if (options.compare === '==') {
        return abstractEquals;
    }

    if (options.compare === '===') {
        return strictEquals;
    }

    var propsRestricted = arePropsRestricted(options);

    var eq = options.strict === false ? abstractCustomizer : null;

    if (propsRestricted && options.by != null) {

        var propsTree = getPropsTree(options.by);

        return function (a, b) {
            return isEqualBy(a, b, eq, propsTree);
        };
    } else if (propsRestricted && options.skip != null) {

        var _propsTree = getPropsTree(options.skip);

        return function (a, b) {
            return isEqualSkip(a, b, eq, _propsTree);
        };
    } else {

        return function (a, b) {
            return isEqualWith(a, b, eq);
        };
    }
}

function abstractEquals(a, b) {
    return a == b;
}

function strictEquals(a, b) {
    return a === b;
}

function arePropsRestricted(options) {
    return (options.by != null || options.skip != null) && options.compare !== '==' && options.compare !== '===' && typeof options.compare !== 'function';
}

function isEqualBy(a, b, eq, tree) {

    if (notObject(a) || notObject(b)) {
        return isEqualWith(a, b, eq);
    }

    for (var key in tree) {

        if (tree[key] === true) {
            if (!isEqualWith(a[key], b[key], eq)) {
                return false;
            }
        }

        if ((hasOwnEnumerable(a, key) ^ hasOwnEnumerable(b, key)) === 1) {
            return false;
        }

        if (!hasOwnEnumerable(a, key)) {
            continue;
        }

        if (!isEqualBy(a[key], b[key], eq, tree[key])) {
            return false;
        }
    }

    return true;
}

function isEqualSkip(a, b, eq, tree) {

    if (!tree) {
        return isEqualWith(a, b, eq);
    }

    if (notObject(a) || notObject(b)) {
        return isEqualWith(a, b, eq);
    }

    for (var key in a) {

        if (tree[key] === true) {
            continue;
        }

        if ((hasOwn(a, key) ^ hasOwnEnumerable(b, key)) === 1) {
            return false;
        }

        if (!hasOwn(a, key)) {
            continue;
        }

        if (!isEqualSkip(a[key], b[key], eq, tree[key])) {
            return false;
        }
    }

    return countNotSkippedKeys(a, tree) === countNotSkippedKeys(b, tree);
}

function countNotSkippedKeys(obj, tree) {

    var nKeys = (0, _keys2.default)(obj).length;

    for (var key in tree) {

        if (tree[key] !== true) {
            continue;
        }

        if (hasOwnEnumerable(obj, key)) {
            nKeys--;
        }
    }

    return nKeys;
}

function abstractCustomizer(a, b) {

    if (notObject(a) && notObject(b)) {

        if (isNaN(a) && isNaN(b)) {
            return true;
        }

        return a == b;
    }
}

function getPropsTree(paths) {

    if (!Array.isArray(paths)) {
        paths = [paths];
    }

    var result = {};

    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
        for (var _iterator2 = (0, _getIterator3.default)(paths), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var path = _step2.value;


            if (!Array.isArray(path)) {
                path = String(path).split('.');
            }

            var pathDepth = path.length;
            var lastKey = pathDepth - 1;

            var pointer = result;

            for (var i = 0; i < pathDepth; i++) {

                var key = path[i];

                if (i === lastKey) {
                    pointer[key] = true;
                } else {
                    pointer = pointer[key] = pointer[key] || {};
                }
            }
        }
    } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
            }
        } finally {
            if (_didIteratorError2) {
                throw _iteratorError2;
            }
        }
    }

    return result;
}

function hasOwn(obj, key) {
    return obj.hasOwnProperty(key);
}

function hasOwnEnumerable(obj, key) {
    return hasOwn(obj, key) && obj.propertyIsEnumerable(key);
}

function notObject(obj) {
    return (typeof obj === 'undefined' ? 'undefined' : (0, _typeof3.default)(obj)) !== 'object' || !obj;
}
module.exports = exports['default'];